# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Requires Python 2.4+ and Openssl 1.0+
#
# Implements parts of RFC 2131, 1541, 1497 and
# http://msdn.microsoft.com/en-us/library/cc227282%28PROT.10%29.aspx
# http://msdn.microsoft.com/en-us/library/cc227259%28PROT.13%29.aspx


import os
import sys

parent = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(parent)

def simple_file_grep(file_path, search_str):
    for line in open(file_path):
         if search_str in line:
                return line

def Mockup(target, name, mock):
    def Decorator(func):
        def Wrapper(*args, **kwargs):
            origin = getattr(target, name)
            setattr(target, name, mock)
            try:
                result = func(*args, **kwargs)
            except:
                raise
            finally:
                setattr(target, name, origin)
            return result
        return Wrapper
    return Decorator

class MockFunc():
    def __init__(self, name='', retval=None):
        self.name = name
        self.retval = retval

    def __call__(*args, **kwargs):
        self = args[0]
        self.args = args[1:]
        self.kwargs = kwargs
        return self.retval

def Dummy():
    pass




#!/usr/bin/env python
#
# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Implements parts of RFC 2131, 1541, 1497 and
# http://msdn.microsoft.com/en-us/library/cc227282%28PROT.10%29.aspx
# http://msdn.microsoft.com/en-us/library/cc227259%28PROT.13%29.aspx
#

import subprocess

"""
WARNING: This script will remove all partitions in resource disk and create
a new one using the entire disk space.
"""
if __name__ == '__main__':
    print 'Unmount resource disk...'
    subprocess.call(['umount', '/dev/sdb1'])
    print 'Remove old partitions...'
    subprocess.call(['parted', '/dev/sdb', 'rm', '1'])
    subprocess.call(['parted', '/dev/sdb', 'rm', '2'])
    print 'Create new partition using the entire resource disk...'
    subprocess.call(['parted', '/dev/sdb','mkpart', 'primary', '0%', '100%'])
    subprocess.call(['mkfs.ext4', '/dev/sdb1'])
    subprocess.call(['mount', '/dev/sdb1', '/mnt'])
    print 'Resource disk(/dev/sdb1) is mounted at /mnt'




#!/usr/bin/env python
#
# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Implements parts of RFC 2131, 1541, 1497 and
# http://msdn.microsoft.com/en-us/library/cc227282%28PROT.10%29.aspx
# http://msdn.microsoft.com/en-us/library/cc227259%28PROT.13%29.aspx
#

import os
from env import waagent
"""
To run the test, you need to create a file under the same directory called:
    status_blob_url.py
and defined the following 2 variables like:
    blockBlobUrl="<sas link to a block blob with w/r access>"
    pageBlobUrl="<sas link to a page blob with w/r access>"
"""
from status_blob_url import blockBlobUrl, pageBlobUrl

class MockConfig(object):
    def get(self, keyName):
        return None

waagent.Config = MockConfig()

if __name__ == '__main__':
    waagent.LoggerInit('/dev/stdout', '/dev/null', verbose=True)
    status = "a" * 512
    waagent.UploadStatusBlob(blockBlobUrl, status.encode("utf-8"))
    #waagent.UploadStatusBlob(pageBlobUrl, status.encode("utf-8"))



#!/usr/bin/env python
#
# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Implements parts of RFC 2131, 1541, 1497 and
# http://msdn.microsoft.com/en-us/library/cc227282%28PROT.10%29.aspx
# http://msdn.microsoft.com/en-us/library/cc227259%28PROT.13%29.aspx
#

import sys
import os
import shutil
import imp
import subprocess
import time
import re
import platform

def upgrade(account='Azure', ref='2.0'):
    #Define variables
    agentUri = ('https://raw.githubusercontent.com/{0}/'
                'WALinuxAgent/{1}/waagent').format(account, ref)
    distro = platform.linux_distribution()
    cmd = ['service', 'waagent', 'restart']
    agent_file="/usr/sbin/waagent"

    if "Ubuntu" in distro[0]:
        cmd[1]='walinuxagent'
    if "CoreOS" in distro[0]:
        cmd = ['systemctl', 'restart', 'waagent']
        agent_file = "/usr/share/oem/bin/waagent"

    if os.path.isfile('waagent'):
        os.remove('waagent')
    print "Download WAAgent from: {0}".format(agentUri)
    try:
        import urllib2
        response = urllib2.urlopen(agentUri)
        html = response.read()
        with open("waagent", "w+") as F:
            F.write(html)
    except:
        subprocess.call(['wget', agentUri])

    print "Upgrade WAAgent"

    shutil.copyfile("waagent", agent_file)
    os.chmod(agent_file, 0755)

    job = subprocess.Popen(cmd)
    job.wait()
    
if __name__ == '__main__':
    if len(sys.argv) == 3:
        upgrade(sys.argv[1], sys.argv[2])
    else:
        upgrade()



# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import unittest
import tempfile
import os
from env import waagent

sample_mount_list = """\
/dev/sda1 on / type ext4 (rw)
proc on /proc type proc (rw)
sysfs on /sys type sysfs (rw)
devpts on /dev/pts type devpts (rw,gid=5,mode=620)
tmpfs on /dev/shm type tmpfs (rw,rootcontext="system_u:object_r:tmpfs_t:s0")
none on /proc/sys/fs/binfmt_misc type binfmt_misc (rw)
/dev/sdb1 on /mnt/resource type ext4 (rw)
"""

device_name="/dev/sdb"

class TestWAAgentUtils(unittest.TestCase):
    
    def test_get_mount_point(self):
        normal = sample_mount_list
        mp = waagent.GetMountPoint(normal, device_name)
        self.assertEqual(mp, '/mnt/resource')

        null = None
        mp = waagent.GetMountPoint(null, device_name)
        self.assertEqual(mp, None)

        malformed = 'asdfasdfasdfa aasdf'
        mp = waagent.GetMountPoint(malformed, device_name)
        self.assertEqual(mp, None)

    def test_replace_in_file_found(self):
        tmpfilename = tempfile.mkstemp('', 'tmp', None, True)[1]
        try:
            tmpfile = open(tmpfilename, 'w')
            tmpfile.write('Replace Me')
            tmpfile.close()

            result = waagent.ReplaceStringInFile(tmpfilename, r'c. ', 'ced ')

            tmpfile = open(tmpfilename, 'r')
            newcontents = tmpfile.read();
            tmpfile.close()

            self.assertEqual('Replaced Me', str(newcontents))
        finally:
            os.remove(tmpfilename)

    def test_replace_in_file_not_found(self):
        tmpfilename = tempfile.mkstemp('', 'tmp', None, True)[1]
        try:
            tmpfile = open(tmpfilename, 'w')
            tmpfile.write('Replace Me')
            tmpfile.close()

            result = waagent.ReplaceStringInFile(tmpfilename, r'not here ', 'ced ')

            tmpfile = open(tmpfilename, 'r')
            newcontents = tmpfile.read();
            tmpfile.close()

            self.assertEqual('Replace Me', str(newcontents))
        finally:
            os.remove(tmpfilename)

if __name__ == '__main__':
    unittest.main()



# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import unittest
from env import waagent


class TestWAAgentLogger(unittest.TestCase):
    
    def test_log_to_non_exists_dev(self):
        logger = waagent.Logger('/tmp/testlog', '/dev/nonexists')
        logger.Log("something")

    def test_log_to_non_exists_file(self):
        logger = waagent.Logger('/tmp/nonexists/testlog', '/tmp/testconsole')
        logger.Log("something")

    def test_log_unicode(self):
        logger = waagent.Logger('/tmp/testlog', '/tmp/testconsole')
        logger.Log(u"anything\u6211\u7231\u5201\u831C".encode("utf-8"))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Implements parts of RFC 2131, 1541, 1497 and
# http://msdn.microsoft.com/en-us/library/cc227282%28PROT.10%29.aspx
# http://msdn.microsoft.com/en-us/library/cc227259%28PROT.13%29.aspx
#

import subprocess

if __name__ == '__main__':
    subprocess.call(['umount', '/mnt/resource'])
    subprocess.call(['umount', '/mnt'])
    subprocess.call(['parted', '/dev/sdb', 'print'])
    subprocess.call(['parted', '/dev/sdb', 'rm', '1'])
    subprocess.call(['parted', '/dev/sdb', 'mklabel', 'gpt'])
    subprocess.call(['parted', '/dev/sdb', 'mkpart', 'primary', '0%', '50%'])
    subprocess.call(['parted', '/dev/sdb', 'mkpart', 'primary', '50%', '100%'])



#!/usr/bin/python

import os
import sys
import platform
import socket
import fcntl
import struct
import array
import re
import tempfile
import unittest
import random
import string
import threading
from time import ctime, sleep
import imp

# waagent has no '.py' therefore create waagent module import manually.
waagent=imp.load_source('waagent','../waagent')

TestingVersion = "$CommitBranch:future$|$LastCommitDate:2013-04-16 15:52:17 -0700$|$LastCommitHash:7ad7c643b2adbac40b1ea4a5b6eb19f0fe971623$"


class WaagentTestCases(unittest.TestCase):
    """
    Test cases for waagent
    """
    def setUp(self):
        """
        Check for root permissions.
        Check Distro is supported.
        Create a waagent.conf file.
        """
        waagent.LoggerInit('/var/log/waagent.log','/dev/console')
        if not self.AmIRoot():
            raise Exception('I need to run as root')
        DistroName=platform.dist()[0]
        self.failUnless(hasattr(waagent,DistroName+'Distro') == True,DistroName+' is not a supported linux distribution.')
        waagent.MyDistro=getattr(waagent,DistroName+'Distro')()
        # set up /etc/waagent.conf
        with open('/etc/waagent.conf','wb') as f:
            f.write(waagent.WaagentConf)
            f.close()
            
    def tearDown(self):
        """
        Remove test resources.
        This is a stub
        """
        pass

    def AmIRoot(self):
        """
        Check that our uid is root.
        """
        return 'root' in waagent.RunGetOutput('id')[1]

    def writetothelog(self,id):
        """
        Convienence function.
        Used by testTwoLogWritingThreads()
        Write 'start', sleep for id seconds and
        write 'end' to the logfile.
        """
        waagent.Log(str(id)+' start ')
        sleep(id)
        waagent.Log(str(id)+' end  ')

    def noop(self,arg2):
        """
        Set a method to noop() to prevent its operation.
        """
        pass

##############TESTCASES##########################

###############Astract Distro - Concrete Distro Tests##############

    def testMyDistroMemberVariables(self):
        """
        Ensure that required Distro properties are not None.
        """
        assert  waagent.MyDistro.agent_service_name is not None , 'MyDistro.agent_service_name must not be None'
        assert  waagent.MyDistro.selinux is not None , 'MyDistro.selinux must not be None'
        assert  waagent.MyDistro.ssh_service_name is not None , 'MyDistro.ssh_service_name must not be None'
        assert  waagent.MyDistro.ssh_config_file is not None , 'MyDistro.ssh_config_file must not be None'
        assert  waagent.MyDistro.hostname_file_path is not None , 'MyDistro.hostname_file_path must not be None'
        assert  waagent.MyDistro.dhcp_client_name is not None , 'MyDistro.dhcp_client_name must not be None'
        assert  waagent.MyDistro.requiredDeps is not None , 'MyDistro.requiredDeps must not be None'
        assert  waagent.MyDistro.init_script_file is not None , 'MyDistro.init_script_file must not be None'
        assert  waagent.MyDistro.agent_package_name is not None , 'MyDistro.agent_package_name must not be None'
        assert  waagent.MyDistro.fileBlackList is not None , 'MyDistro.fileBlackList must not be None'
        assert  waagent.MyDistro.agent_files_to_uninstall is not None , 'MyDistro.agent_files_to_uninstall must not be None'
        assert  waagent.MyDistro.grubKernelBootOptionsFile is not None , 'MyDistro.grubKernelBootOptionsFile must not be None'
        assert  waagent.MyDistro.grubKernelBootOptionsLine is not None , 'MyDistro.grubKernelBootOptionsLine must not be None'


    def testMyDistro_restartSshService(self):
        """
        Test MyDistro.restartSshService()
        """
        cmd = 'service '+ waagent.MyDistro.ssh_service_name + ' status'
        sshpid=string.rsplit(waagent.RunGetOutput(cmd)[1],' ',1)
        waagent.MyDistro.restartSshService()
        assert sshpid is not string.rsplit(waagent.RunGetOutput(cmd)[1],' ',1),'ssh server pid is unchanged.'

#     def testMyDistro_checkPackageInstalled(self):
#         """MyDistro can check if WaLinuxAgent package is installed"""
#         assert waagent.MyDistro.checkPackageInstalled(waagent.MyDistro.agent_package_name) != 0, waagent.MyDistro.agent_package_name+' is Not Installed.'

#     def testMyDistro_checkPackageUpdateable(self):
#         """MyDistro can check if WaLinuxAgent package is updateable to new version."""
#         assert waagent.MyDistro.checkPackageUpdateable(waagent.MyDistro.agent_package_name) == 0 , waagent.MyDistro.agent_package_name+' is not updateable.'

    
    def testMyDistro_isSelinuxSystem(self):
        """
        MyDistro can perform Selinux operations.
        Test MyDistro.isSelinuxSystem, if true then also test:
        MyDistro.isSelinuxRunning
        MyDistro.setSelinuxEnforce
        MyDistro.setSelinuxContext
        """
        selinux=waagent.MyDistro.isSelinuxSystem()
        if selinux:
            assert waagent.MyDistro.isSelinuxRunning(), 'Selinux not running.'
            assert waagent.MyDistro.setSelinuxEnforce(0), 'Unable to call setenforce(0).'
            assert waagent.MyDistro.setSelinuxContext('./test_waagent.py','unconfined_u:object_r:ssh_home_t:s0'), 'Unable to set Selinux context.'
            assert waagent.MyDistro.setSelinuxEnforce(0), 'Unable to call setenforce(1).'
        else:
            print 'Selinux not installed. - skipping Selinux tests'
        
    def testMyDistro_load_unload_ata_piix(self):
        """
        Attempt to insert and remove ata_piix.ko
        by calling MyDistro.load_ata_piix
        and MyDistro.unload_ata_piix.
        """
        assert waagent.MyDistro.load_ata_piix() == 0, 'Unable to load ata_piix.ko.'
        assert waagent.MyDistro.unload_ata_piix() == 0, 'Unable to unload ata_piix.ko.'
        
    def testMyDistro_publishHostname(self):
        """
        Test MyDistro.publishHostname
        on success, the distro dependent config
        contains the hostname, but currently
        this test suceeds if the config files were written
        without error.
        """
        assert waagent.MyDistro.publishHostname('LENG') == 0, 'Error setting hostname to LENG.'

#     def testMyDistro_registerAgentService(self):
#         assert waagent.MyDistro.registerAgentService() == 0, 'Unable to register agent as service.'

    def testMyDistro_setHostname(self):
        """
        Test MyDistro.setHostname.
        Successfull if hostname is changed.
        Reset hostname when finished.
        """
        code,oldname = waagent.RunGetOutput('hostname')
        waagent.MyDistro.setHostname('HOSTNAMETEST')
        code,newname = waagent.RunGetOutput('hostname')
        assert 'HOSTNAMETEST' == newname.strip(), 'Unable to set hostname.'
        waagent.MyDistro.setHostname(oldname)

    def testMyDistro_checkDependencies(self):
        """
        Test MyDistro.checkDependencies succeeds
        """
        assert waagent.MyDistro.checkDependencies() == 0 , 'Dependency Check failed.'
        
    def testMyDistro_startAgentService(self):
        """
        Test MyDistro.startAgentService.
        """
        assert waagent.MyDistro.startAgentService() == 0, 'Unable to start ' + waagent.MyDistro.agent_service_name

    def testMyDistro_stopAgentService(self):
        """
        Test MyDistro.stopAgentService.
        """
        assert waagent.MyDistro.stopAgentService() == 0, 'Unable to stop ' + waagent.MyDistro.agent_service_name
         
    def testMyDistro_deleteRootPassword(self):
        """
        Test MyDistro.deleteRootPassword.
        Restore the shadow file to previous state when finished.
        """
        #copy the shadow
        waagent.Run('cp /etc/shadow /etc/shadow.keep')
        waagent.MyDistro.deleteRootPassword()
        assert waagent.Run('grep LOCK /etc/shadow') == 0 , 'Error removing root password.'
        # put shadow back 
        waagent.Run('mv /etc/shadow.keep /etc/shadow')
        
    def testFindIn_AppendTo_RemoveFrom_LinuxKernelCmdline(self):
        """
        Test LinuxKernelCmdline operations.
        Search for 'splish=splash' in the kernel boot options, expect fail.
        Add 'splish=splash'.  Search for splish=splash expect success.
        Remove 'splish=splash', confirm splish=splash absent
        """
        m=waagent.FindInLinuxKernelCmdline('splish=splash')
        assert not m, '"splish=splash" was found before i put it there!!! edit it to remove "splish=splash" please.'
        
        waagent.AppendToLinuxKernelCmdline('splish=splash')
        m=waagent.FindInLinuxKernelCmdline('splish=splash')
        assert m, 'AppendToLinuxKernelCmdline failed, "splish=splash" still not found.'

        waagent.RemoveFromLinuxKernelCmdline('splish=splash')
        m=waagent.FindInLinuxKernelCmdline('splish=splash')
        assert not m, 'RemoveFromLinuxKernelCmdline failed, "splish=splash" still found.'

###############Generic waagent tests##############

    def testLogFile(self):
        """
        Write a random number with waagent.Log() and read it back.
        """
        rnds=str(random.random())
        waagent.Log('testLogFile: '+rnds)
        found = rnds in (open('/var/log/waagent.log','rb').read())
        assert found,'Unable to find '+rnds+' in /var/log/waagent.log'
        
    def testFindReplaceStringInFile(self):
        """
        Test file/string operations using
        string literals and regular expressions.
        Tests:
        FindStringInFile
        ReplaceStringInFile
        
        """
        fn='/tmp/junk'
        if os.path.exists(fn):
            os.remove(fn)
        sp='splish splash'
        yb='yabba dabba do'
        open(fn,'wb').write(sp+' I was taking a bath.')
        m=waagent.FindStringInFile(fn,sp)
        assert m is not None,'waagent.FindStringInFile() Failed: '+sp+' not found in ' + fn + '.'
        src=r'^(.*)('+sp+')(.*)$'
        rpl=r'\1 '+sp+'\2 '+yb+' \3'
        waagent.ReplaceStringInFile(fn,src,rpl)
        m=waagent.FindStringInFile(fn,yb)
        assert m is not None,'waagent.ReplaceStringInFile() Failed: '+yb+' not found in ' + fn + '.'

    def testGetFirstActiveNetworkInterfaceNonLoopback(self):
        """
        Test GetFirstActiveNetworkInterfaceNonLoopback.
        Fail if iface is 'lo'
        """
        addr='null'
        iface=waagent.GetFirstActiveNetworkInterfaceNonLoopback()[0]
        addr=waagent.GetFirstActiveNetworkInterfaceNonLoopback()[1]
        assert len(iface)>1,'Interface name too short'
        assert iface is not 'lo','Loopback Interface was returned'
        print 'iface=' + iface + ' addr=' + addr

    def testTwoLogWritingThreads(self):
        """
        Test that two threads writing to the same log
        function do not block or scramble messages.
        TODO - there is no check for success !!!
        """
        for j in range(5):
            t1=threading.Thread(target=self.writetothelog,args=(4,))
            t2=threading.Thread(target=self.writetothelog,args=(2,))
            t1.start()
            t2.start()
            t1.join()
            t2.join()

    def testCertificatesParse(self):
        """
        TODO - need cert xml from test...
        """
        pass
    
    def testSharedConfigParse(self):
        
        """
        Test SharedConfig().Parse returns without error.
        """
        assert waagent.SharedConfig().Parse(SHAREDCONFIG), 'Error parsing SharedConfig.xml'


    def testOvfEnvParse(self):
        """
        Test OvfEnv().Parse returns without error.
        """
        assert waagent.OvfEnv().Parse(OVFXML) is not None , 'Failed to Parse ovfxml'

    def testOvfEnvProcess(self):
        """
        We expect the /var/lib/waagent/Certificates.p7m file exists.
        Test ovfenv.Process() return other than None.
        """
        assert os.path.exists('/var/lib/waagent/Certificates.p7m') , 'We expect the /var/lib/waagent/Certificates.p7m file exists.'
        waagent.WaAgent = waagent.Agent()
        ovfenv=waagent.OvfEnv().Parse(OVFXML)
        waagent.WaAgent.EnvMonitor = waagent.EnvMonitor()
        assert ovfenv.Process() is None , 'Failed to Process ovfxml'
        waagent.Run("userdel -f -r myUserName")
        
    def testAgentProvision(self):
        """
        TODO - Test Provision in non-fabric environment
        """
        waagent.verbose = True
        waagent.WaAgent = waagent.Agent()
        waagent.WaAgent.EnvMonitor = waagent.EnvMonitor()
        waagent.Config = waagent.ConfigurationProvider()
        # we cant report our role unless we have one.
        waagent.WaAgent.ReportRoleProperties=self.noop
        err=waagent.WaAgent.Provision()
        assert err == None, 'Provision Failed error ' + str(err)
        

########################################
    


OVFXML="""<?xml version="1.0" encoding="utf-8"?>
<Environment xmlns="http://schemas.dmtf.org/ovf/environment/1" xmlns:oe="http://schemas.dmtf.org/ovf/environment/1" xmlns:wa="http://schemas.microsoft.com/windowsazure" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">

    <wa:ProvisioningSection><wa:Version>1.0</wa:Version><LinuxProvisioningConfigurationSet xmlns="http://schemas.microsoft.com/windowsazure" xmlns:i="http://www.w3.org/2001/XMLSchema-instance"><ConfigurationSetType>LinuxProvisioningConfiguration</ConfigurationSetType><HostName>egub13-vm</HostName><UserName>myUserName</UserName><UserPassword>mypassword</UserPassword><DisableSshPasswordAuthentication>false</DisableSshPasswordAuthentication><SSH><PublicKeys><PublicKey><Fingerprint>2D97B25D49B98ECC90BF1600D66D68799CFB361E</Fingerprint><Path>/home/myUserName/.ssh/authorized_keys</Path></PublicKey></PublicKeys></SSH></LinuxProvisioningConfigurationSet></wa:ProvisioningSection>

  <wa:PlatformSettingsSection><wa:Version>1.0</wa:Version><PlatformSettings xmlns="http://schemas.microsoft.com/windowsazure" xmlns:i="http://www.w3.org/2001/XMLSchema-instance"><KmsServerHostname>kms.core.windows.net</KmsServerHostname></PlatformSettings></wa:PlatformSettingsSection>
</Environment>
"""

SHAREDCONFIG="""
<SharedConfig version="1.0.0.0" goalStateIncarnation="1">
  <Deployment name="db00a7755a5e4e8a8fe4b19bc3b330c3" guid="{ce5a036f-5c93-40e7-8adf-2613631008ab}" incarnation="2">
    <Service name="MyVMRoleService" guid="{00000000-0000-0000-0000-000000000000}" />
    <ServiceInstance name="db00a7755a5e4e8a8fe4b19bc3b330c3.1" guid="{d113f4d7-9ead-4e73-b715-b724b5b7842c}" />
  </Deployment>
  <Incarnation number="1" instance="MachineRole_IN_0" guid="{a0faca35-52e5-4ec7-8fd1-63d2bc107d9b}" />
  <Role guid="{73d95f1c-6472-e58e-7a1a-523554e11d46}" name="MachineRole" settleTimeSeconds="10" />
  <LoadBalancerSettings timeoutSeconds="0" waitLoadBalancerProbeCount="8">
    <Probes>
      <Probe name="MachineRole" />
      <Probe name="55B17C5E41A1E1E8FA991CF80FAC8E55" />
      <Probe name="3EA4DBC19418F0A766A4C19D431FA45F" />
    </Probes>
  </LoadBalancerSettings>
  <OutputEndpoints>
    <Endpoint name="MachineRole:Microsoft.WindowsAzure.Plugins.RemoteAccess.Rdp" type="SFS">
      <Target instance="MachineRole_IN_0" endpoint="Microsoft.WindowsAzure.Plugins.RemoteAccess.Rdp" />
    </Endpoint>
  </OutputEndpoints>
  <Instances>
    <Instance id="MachineRole_IN_0" address="10.115.153.75">
      <FaultDomains randomId="0" updateId="0" updateCount="0" />
      <InputEndpoints>
        <Endpoint name="a" address="10.115.153.75:80" protocol="http" isPublic="true" loadBalancedPublicAddress="70.37.106.197:80" enableDirectServerReturn="false" isDirectAddress="false" disableStealthMode="false">
          <LocalPorts>
            <LocalPortRange from="80" to="80" />
          </LocalPorts>
        </Endpoint>
        <Endpoint name="Microsoft.WindowsAzure.Plugins.RemoteAccess.Rdp" address="10.115.153.75:3389" protocol="tcp" isPublic="false" enableDirectServerReturn="false" isDirectAddress="false" disableStealthMode="false">
          <LocalPorts>
            <LocalPortRange from="3389" to="3389" />
          </LocalPorts>
        </Endpoint>
        <Endpoint name="Microsoft.WindowsAzure.Plugins.RemoteForwarder.RdpInput" address="10.115.153.75:20000" protocol="tcp" isPublic="true" loadBalancedPublicAddress="70.37.106.197:3389" enableDirectServerReturn="false" isDirectAddress="false" disableStealthMode="false">
          <LocalPorts>
            <LocalPortRange from="20000" to="20000" />
          </LocalPorts>
        </Endpoint>
      </InputEndpoints>
    </Instance>
  </Instances>
</SharedConfig>
"""


if __name__ == '__main__':
    s=unittest.TestLoader().loadTestsFromTestCase(WaagentTestCases)
    unittest.TextTestRunner(verbosity=2).run(s)
    #    import cProfile
    #    cProfile.run('unittest.TextTestRunner(verbosity=2).run(s)','profile.out')




# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import unittest
from env import waagent
import sys
from tests.tools import *

SampleInterfaceInfo="""\
eth0      Link encap:Ethernet  HWaddr ff:ff:ff:ff:ff:ff  
          inet addr:10.94.20.249  Bcast:10.94.23.255  Mask:255.255.252.0
          inet6 addr: fe80::215:5dff:fe5f:bf03/64 Scope:Link
          UP BROADCAST RUNNING MULTICAST  MTU:1500  Metric:1
          RX packets:3789880 errors:0 dropped:0 overruns:0 frame:0
          TX packets:80973 errors:0 dropped:0 overruns:0 carrier:0
          collisions:0 txqueuelen:1000 
          RX bytes:388563383 (388.5 MB)  TX bytes:21484571 (21.4 MB)

eth1      Link encap:Ethernet  HWaddr 00:00:00:00:00:00  
          inet addr:192.168.1.1  Bcast:192.168.1.255  Mask:255.255.255.0
          inet6 addr: fe80::215:5dff:fe5f:bf08/64 Scope:Link
          UP BROADCAST RUNNING MULTICAST  MTU:1500  Metric:1
          RX packets:386614 errors:0 dropped:0 overruns:0 frame:0
          TX packets:201356 errors:0 dropped:0 overruns:0 carrier:0
          collisions:0 txqueuelen:1000 
          RX bytes:32507619 (32.5 MB)  TX bytes:78342503 (78.3 MB)

lo        Link encap:Local Loopback  
          inet addr:127.0.0.1  Mask:255.0.0.0
          inet6 addr: ::1/128 Scope:Host
          UP LOOPBACK RUNNING  MTU:65536  Metric:1
          RX packets:2561 errors:0 dropped:0 overruns:0 frame:0
          TX packets:2561 errors:0 dropped:0 overruns:0 carrier:0
          collisions:0 txqueuelen:0 
"""

class TestUtil(unittest.TestCase):

    @Mockup(waagent, "RunGetOutput", MockFunc('', (0, SampleInterfaceInfo)))
    def test_getInterfaceNameByMac(self):
        distro = waagent.AbstractDistro()
        ifName = distro.getInterfaceNameByMac("ff:ff:ff:ff:ff:ff")
        self.assertEquals("eth0", ifName)
        ifName = distro.getInterfaceNameByMac("00:00:00:00:00:00")
        self.assertEquals("eth1", ifName)
        

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import os
import sys
import imp
import json
import time
import pwd

# waagent has no '.py' therefore create waagent module import manually.
waagent=imp.load_source('waagent','waagent')

from waagent import RunGetOutput,  Run, LoggerInit

"""
Test waagent in azure using azure-cli
Usage:

./azure_test.py --stable_vm_image b4590d9e3ed742e4a1d46e5424aa335e__openSUSE-12.3-v120 --source_disk "http://mystorage.blob.core.windows.net/vhds/my-suse2.vhd" --acct "<storage acct key>" --testname my-osuse --mount_point /mnt/disk --agent_path ../waagent --stable_vm_acct_name MyUserName --stable_vm_acct_pass 'myp455wd' --test_vm_acct_name MyUserName --test_vm_acct_pass 'myp455wd' --azure_location "East US" --part_num 1 --retries 20 --fstype scsi --test_vm_acct_cert /root/.ssh/myCert.pem --stable_vm_acct_cert /root/.ssh/myCert.pem --keep_test_vm_vhd no --teardown_test_vm always --prompt no

azure_test --vm <stable vm name> --testname <testname> [--acct <storage account>]
[--disk <vhd url to use as initial disk image>]
If --disk is specified, use this vhd as starting point,
otherwise use a copy of the stable vm as the starting vhd.

Starting VHD is attached to stable vm and the sources are copied to it.
Spin up a new VM using the VHD.
Loop waiting for provisioned.
If not provisioned:
    Destroy vm and attach the disk to the stable vm.
    Copy the logs to the localhost.
    Destroy all created objects except the starting vhd.
    Exit(1)
If Provosioned:
    Copy the logs to the local host.
    Exit(0)
    
EXAMPLE:

sudo ./azure_test.py --vm my-stable-vm-name --disk "http://mystorageaccount.blob.core.windows.net/myvhds/my-new-os.vhd" --acct mylong-unquoted-starage-account-id --testname my-vm-test --mount_point /my-stablevm-mountpoint --agent_path ../waagent --vm_acct_name root --testvm_acct_name azureuser --testvm_acct_pass 'azureuserpassword' --location "East US" --part_num 2 --retry 20 --fstype bsd --testvm_acct_cert /home/azureuser/.ssh/myCert.pem --keep_vhd "once" --teardown "always" --prompt "no"   
"""

def makeDiskImage(di_name,vhd_url,location,copy=False):
    """
    Create new data disk image of the VHD.  If 'copy'
    is set to True, then create a new VHD in the form of myvhd-di.vhd
    based on the VHD source name.  If 'copy is set to False, re-use the
    vhd.  Returns return code and diskimageVHD path.  Code is 0 on
    success or the azure-cli error code upon error.
    """
    if copy :
        target = os.path.dirname(vhd_url)
        target = target+ '/' + di_name +  '.vhd'
    else :
        target = vhd_url
    cmd='azure vm disk create --json ' + di_name + ' --blob-url ' + target +  ' ' + vhd_url
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log(output)
    waagent.Log(str(code))
    return target,code

def makeVMImage(vmi_name,vhd_url,copy=False):
    """
    Create new VM Image based on Disk Image.
    Returns 0 on success or error code upon error.
    """
    if copy :
        target = os.path.dirname(vhd_url)
        target = target+ '/' + vmi_name +  '.vhd'
    else :
        target = vhd_url
    cmd='azure vm image create --json ' + vmi_name + ' --base-vhd '  + target + ' --os Linux --blob-url ' + vhd_url
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log(str(code))
    waagent.Log(output)
    return code

def makeVM(vm_name,vmi_name,vhd_url,test_name,location,vmuser_name,vmuser_pass,vmuser_cert,copy=False):
    """
    Create new VM from the VM Image.
    Returns 0 on success or error code upon error.
    """
    target=os.path.dirname(vhd_url)
    target = target + '/' + test_name +  '.vhd'
    cmd='azure vm create --json '
    if copy :
        cmd += ' --blob-url "' + target + '"'
    else :
        target=vhd_url
    cmd += ' --location "' + location + '"'
    if os.path.exists(vmuser_cert):
        cmd += ' -t "' + vmuser_cert + '"'
    cmd += ' -e 22 ' + vm_name + ' ' + vmi_name + ' ' + vmuser_name + ' \'' +vmuser_pass + '\''
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log(str(code))
    waagent.Log(output)
    retry=3
    while code !=0 and retry > 0 :
        time.sleep(5)
        code,output=RunGetOutput(cmd,False)
        retry -=1
    return target

def flushDiskImage(di_name,dele=True):
    """
    Delete the VM Image.
    On error we asume the VM disk image is deleted
    """
    cmd='azure vm disk delete --json '
    if dele :
        cmd += '--blob-delete '
    cmd+= di_name
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log( str(code))
    waagent.Log(output)
    return output,code

def flushVMImage(vmi_name):
    """
    Delete the VM Image.
    Always delete the underlying blob.
    On error we asume the VM image is deleted.
    """
    cmd='azure vm image delete --blob-delete --json ' + vmi_name
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log( str(code))
    waagent.Log(output)
    return output,code

def flushVM(vm_name,dele=False):
    """
    Delete the VM.
    On error we asume the VM is deleted
    """
    cmd='azure vm delete --json '
    if dele :
        cmd += ' --blob-delete '
    cmd += vm_name
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log( str(code)) 
    waagent.Log(output)
    return output,code


def createStableVMFromVMImage(vm_image,vhd_url):
    """
    Create a new stable vm, provisioned with acct and certificate from
    the VMImage, using the basepath of vhd_url for the new VM's vhd.
    """
    stableVM=testname+'-stable-vm'
    return makeVM(stableVM,vm_image,vhd_url,testname+'-stable',location,stableVMaccount,stableVMpass,stableVMCert,copy=True)
    
def createStableVMFromVHD(vmi_name,vhd_url):
    """
    Create a new stable vm, provisioned with acct and certificate from
    the VHD, using the basepath of vhd_url for the new VM's vhd.
    """
    makeVMImage(vmi_name,vhd_url,False)
    return createStableVMFromVMImage(vmi_name,vhd_url)

def createDiskImageFromStableVMDisk(vm_name):
    """
    Determine the media link for the os disk of the stable vm.
    Create vhd disk image copy.  <vm_name>-<testname>-di
    Return new disk_image_media_path or None on error.
    """
    cmd='azure vm disk list --json'
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log( str(code))
    waagent.Log(output)
    if code:
        print 'Error is ' + str(code)
        waagent.Log( 'Error is ' + str(code))
        return None
    j=json.loads(output)
    source_media_link=None
    for i in j :
        if i.has_key('AttachedTo'):
            if i['AttachedTo']['RoleName'] == vm_name:
                source_media_link=i['MediaLink']
                break
    if not source_media_link:
        print 'Unable to locate OS disk for ' + vm_name 
        waagent.Log( 'Unable to locate OS disk for ' + vm_name )
        return None
    target_name= testname + '-di'
    makeDiskImage(target_name,source_media_link,location,copy=True)
    target_media_link=os.path.dirname(source_media_link) + '/' + target_name + '.vhd'
    return target_media_link

def addDiskImageToVM(vm_name,di_name):
    """
    Attach the disk image to the 'stableVM'.
    Returns the LUN if successful otherwise returns None
    NOTE: azure vm show may return json matching the disk image
    name yet missing the LUN.  When this occurs, the LUN is '0'. 
    """
    cmd='azure vm disk attach --json ' + vm_name + ' ' + di_name
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log(str(code))
    waagent.Log(output)
    cmd='azure vm show --json ' + vm_name
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log( str(code))
    waagent.Log(output)
    retries=3
    while code != 0 and retries: 
        retries-=1
        print cmd
        waagent.Log( cmd)
        code,output=RunGetOutput(cmd,False)
        
    if code == 0:
        jsn=json.loads(output)
        for i in jsn['DataDisks']:
            if i['DiskName'] == di_name:
                if 'Lun' in i :
                    return i['Lun']
                else :
                    return u'0'
    return None

    
def dropDiskImageFromVM(vm_name,lun):
    """
    Detach the disk image from the 'stableVM'.
    On Error we assume the disk is no longer attached.
    """
    cmd='azure vm disk detach --json ' + vm_name + ' ' + lun
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log( str(code))
    waagent.Log(output)
    return output,code

def checkVMProvisioned(vm_name):
    cmd='azure vm show --json ' + vm_name
    print cmd
    waagent.Log( cmd)
    code,output=RunGetOutput(cmd,False)
    print output,code
    waagent.Log( str(code))
    waagent.Log(output)
    if code ==0 :
        j=json.loads(output)
        print vm_name+' instance status: ', j['InstanceStatus']
        waagent.Log( vm_name+' instance status: ' + j['InstanceStatus'])
        if j['InstanceStatus'] == 'ReadyRole':
            return True, j['InstanceStatus']
    else :
        print 'Error: ' + output , code
        waagent.Log( 'Error: ' + output +  str(code))
    return False, j['InstanceStatus']
        
def updateAgent(agent_path,vm_name,account,cert,disk_mountpoint,mnt_opts,lun,partnum,provisioned_account):
    """
    Copy the agent specified in 'agent' to the Disk
    using the 'stableVM'.
    """
    retries=30
    retry=0
    cmd='uptime'
    while  ssh_command(vm_name,account,cmd)[1] != 0 and  retry < retries :
        time.sleep(10)
        retry+=1
    #setup sudo NOPASSWD
    pss=stableVMpass.replace('$','\$')
    cmd='echo \'' + pss  + '\' > /home/' + account + '/pswd '
    ssh_command(vm_name,account,cmd)
    cmd='echo \'#!/bin/bash\ncat /home/' + account + '/pswd\n\' > /home/' + account + '/pw.sh'
    ssh_command(vm_name,account,cmd)
    cmd='chmod +x /home/' + account + '/pw.sh'
    ssh_command(vm_name,account,cmd)
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A mkdir -p ' + disk_mountpoint
    ssh_command(vm_name,account,cmd)
    retries=3
    # TODO retires here for the mount
    #mount
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A mount ' +mnt_opts + ' ' + lunToDiskName(lun,partnum) + ' ' +disk_mountpoint
    waagent.Log( cmd)
    retry=0
    while  ssh_command(vm_name,account,cmd)[1] not in (0,32) and  retry < retries :
        if retry == 0:
            if 'bsd' in fstype:
                fcmd = "export SUDO_ASKPASS=./pw.sh && sudo -A fsck_ffs -y "
            else :
                fcmd = "export SUDO_ASKPASS=./pw.sh && sudo -A fsck -y "
            fcmd += lunToDiskName(lun,partnum)
            ssh_command(vm_name,account,fcmd)
        time.sleep(2)
        retry+=1

    # remove packaged agent service if present.
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A chroot '+ disk_mountpoint+' dpkg -r walinuxagent'
    ssh_command(vm_name,account,cmd)    # remove Ubuntu walinuxagent agent service if present.
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A rm '+ disk_mountpoint+'/etc/default/walinuxagent'
    ssh_command(vm_name,account,cmd)    # remove Ubuntu walinuxagent agent service if present.
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A chroot '+ disk_mountpoint+' rpm -e WALinuxAgent'
    ssh_command(vm_name,account,cmd)
    #copy agent
    remote_path='/tmp'
    print 'scp ' + agent_path + ' to ' + vm_name + ' ' + account + ':' + remote_path
    waagent.Log( 'scp ' + agent_path + ' to ' + vm_name + ' ' + account + ':' + remote_path)
    retry=0
    while scp_to_host_command(account,vm_name,remote_path,agent_path)[1] != 0 and retry < retries :
        time.sleep(2)
        retry+=1
    # move agent to /usr/sbin
    cmd= 'export SUDO_ASKPASS=./pw.sh && sudo -A cp ' + remote_path +'/waagent '+ disk_mountpoint+'/usr/sbin/waagent'
    ssh_command(vm_name,account,cmd)
    cmd= 'export SUDO_ASKPASS=./pw.sh && sudo -A chmod 755 '+ disk_mountpoint+'/usr/sbin/waagent'
    ssh_command(vm_name,account,cmd)
    # Fix the password file
    if 'bsd' in fstype:
        cmd='export SUDO_ASKPASS=./pw.sh && sudo -A cp /etc/master.passwd ' + disk_mountpoint + '/etc/master.passwd'
        ssh_command(vm_name,account,cmd)
    else :
        cmd='export SUDO_ASKPASS=./pw.sh && sudo -A cp /etc/passwd ' + disk_mountpoint + '/etc/passwd'
        ssh_command(vm_name,account,cmd)
        cmd='export SUDO_ASKPASS=./pw.sh && sudo -A cp /etc/shadow ' + disk_mountpoint + '/etc/shadow'
        ssh_command(vm_name,account,cmd)
    #remove /var/lib/waagent
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A rm -rf ' + disk_mountpoint + '/var/lib/waagent'
    ssh_command(vm_name,account,cmd)
    #remove /var/log/waagent*
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A rm -rf ' + disk_mountpoint + '/var/log/waagent*'
    ssh_command(vm_name,account,cmd)
    #delete the provisioning user
    if 'bsd' in fstype:
        cmd='export SUDO_ASKPASS=./pw.sh && sudo -A chroot '+ disk_mountpoint+' rmuser -y ' + provisioned_account
        ssh_command(vm_name,account,cmd)
    else :
        cmd='export SUDO_ASKPASS=./pw.sh && sudo -A chroot '+ disk_mountpoint+' userdel -f ' + provisioned_account
        ssh_command(vm_name,account,cmd)
        cmd='export SUDO_ASKPASS=./pw.sh && sudo -A chroot '+ disk_mountpoint+' groupdel ' + provisioned_account
        ssh_command(vm_name,account,cmd)
        cmd='export SUDO_ASKPASS=./pw.sh && sudo -A rm -rf ' + disk_mountpoint + '/home/' + provisioned_account 
        ssh_command(vm_name,account,cmd)
    # install agent
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A chroot  '+ disk_mountpoint+'  /usr/sbin/waagent verbose install '
    ssh_command(vm_name,account,cmd)
    cmd="export SUDO_ASKPASS=./pw.sh && sudo -A  sed -i 's/Verbose=n/Verbose=y/' " + disk_mountpoint+"/etc/waagent.conf"
    ssh_command(vm_name,account,cmd)
    #umount
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A umount ' + lunToDiskName(lun,partnum)
    ssh_command(vm_name,account,cmd)
    
def gatherAgentInfo(localpath,vm_name,account,cert,disk_mountpoint,mnt_opts,lun,partnum):
    """
    Copy the /var/lib/waagent, and /var/log directories to
    localhost:localpath.
    """
    retries=30
    retry=0
    cmd='uptime'
    while  ssh_command(vm_name,account,cmd)[1] != 0 and  retry < retries :
        time.sleep(10)
        retry+=1
    #mount
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A mount ' +mnt_opts + ' '  + lunToDiskName(lun,partnum) + ' ' +disk_mountpoint
    print cmd
    waagent.Log( cmd)
    ssh_command(vm_name,account,cmd)
    #copy info
    Run("mkdir -p "+ localpath)
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A mkdir -p /tmp/results'
    ssh_command(vm_name,account,cmd)
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A cp -r  ' + disk_mountpoint + '/var/log /tmp/results/'
    ssh_command(vm_name,account,cmd)
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A cp -r  ' + disk_mountpoint + '/var/lib/waagent /tmp/results/'
    ssh_command(vm_name,account,cmd)
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A chmod -R 777 /tmp/results'
    ssh_command(vm_name,account,cmd)
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A chown -R ' + account + '  /tmp/results'
    ssh_command(vm_name,account,cmd)
    scp_from_host_command(account,vm_name,'/tmp/results/*',localpath)
    #umount
    cmd='export SUDO_ASKPASS=./pw.sh && sudo -A umount ' + lunToDiskName(lun,partnum)
    print cmd
    waagent.Log( cmd)
    ssh_command(vm_name,account,cmd)

def lunToDiskName(lun,partnum):
    if 'bsd' in fstype :
        return lunToFreeBSDDiskName(lun,partnum)
    else :
        return lunToScsiDiskName(lun,partnum)

def lunToScsiDiskName(lun,partnum):
    """
    Convert lun to '/dev/sd[chr(ord('c')+lun)]partnum'
    """
    return str('/dev/sd'+chr( (ord('c')+int(lun)) ) +str(partnum))

def lunToFreeBSDDiskName(lun,partnum):
    """
    Convert lun to '/dev/da' + str(lun) + 'p' + partnum
    """
    return '/dev/da'+ str(int(lun))  + 'p' + str(partnum)

def ssh_command(host,account,cmd):
    """
    Wrapper for an ssh operation.
    """
    if stableVMCert == None:
        if not os.path.exists('./pw.sh'):
            with open('./pw.sh','w') as F:
                F.write('#!/bin/bash\ncat ./pswd\n')
            os.system('chmod +x ./pw.sh')
            with open('./pswd','w') as F:
                F.write(stableVMpass)
        req = "export SSH_ASKPASS=./pw.sh && setsid ssh -T -o StrictHostKeyChecking='no' " + account + "@" + host.lower() + ".cloudapp.net \"" + cmd + "\""
    else :
        req = "ssh -t -o StrictHostKeyChecking='no' " + account + "@" + host.lower() + ".cloudapp.net \"" + cmd + "\""
    print req
    waagent.Log(req)
    code,output=RunGetOutput(req,False)
    print output,code
    waagent.Log(str(code))
    waagent.Log(output.encode('ascii','ignore'))
    return output,code

def scp_to_host_command(account,host,remote_path,local_path):
    """
    Wrapper for an scp operation.  Always uses -r.
    Requires key authentication configured.
    """
    req="scp -o StrictHostKeyChecking='no' -r " + local_path + " " + account + "@" + host.lower() + ".cloudapp.net:" + remote_path
    print req
    waagent.Log( req)
    code,output=RunGetOutput(req,False)
    print output,code
    waagent.Log( str(code))
    waagent.Log(output)
    return output,code

def scp_from_host_command(account,host,remote_path,local_path):
    """
    Wrapper for an scp operation.  Always uses -r.
    Requires key authentication configured.
    """
    req="scp -r " + account + "@" + host.lower() + ".cloudapp.net:" + remote_path + " " + local_path
    print req
    waagent.Log( req)
    code,output=RunGetOutput(req,False)
    print output,code
    waagent.Log( str(code))
    waagent.Log(output)
    return output,code

def teardown(name):
    diskImageName=os.path.splitext(os.path.basename(sourceVHD))[0]+'-di'
    while makeDiskImage(diskImageName,sourceVHD,location,True)[1] !=0 :
        time.sleep(20)
    lun=addDiskImageToVM(stableVM,diskImageName)
    while lun == None :
        time.sleep(2)
        lun=addDiskImageToVM(stableVM,diskImageName)
    out,code=flushVM(vmName,True)
    if code != 0 :
        vmDisk=out[out.find('disk with name ')+len('disk with name '):out.find(' is currently in use')]
        while flushDiskImage(vmDisk,True)[1] != 0 :
            time.sleep(5)
    out,code=flushVMImage(vmImageName)
    if code != 0 :
        vmDisk=out[out.find('disk with name ')+len('disk with name '):out.find(' is currently in use')]
        while flushDiskImage(vmDisk,True)[1] != 0 :
            time.sleep(5)
    gatherAgentInfo(localInfo+'/'+name,stableVM,stableVMaccount,stableVMCert,stableVMMountpoint,mountOptions,lun,partNum)
    print 'Logs for ' + vmName + ' copied to ' + localInfo + '/' + name
    waagent.Log( 'Logs for ' + vmName + ' copied to ' + localInfo + '/' + name)
    # detach and delete the disk image
    while dropDiskImageFromVM(stableVM,lun)[1] != 0 :
        time.sleep(2)
    while flushDiskImage(diskImageName,('no' in keep_vhd))[1] != 0 :
        time.sleep(2)
    out,code=flushVM(stableVM,True)
    if code != 0 :
        stableVMDisk=out[out.find('disk with name ')+len('disk with name '):out.find(' is currently in use')]
        while flushDiskImage(stableVMDisk,True)[1] != 0 :
            time.sleep(5)
    if stableVMImageName:
        while flushVMImage(stableVMImageName,True)[1] != 0 :
            time.sleep(2)
    
def doPrompt() :
    if prompt in ('yes','on'):
        raw_input('Press enter to continue:')

if __name__ == '__main__' :
    """
    Create a disk image and attach it to StableVM.
    Copy the current sources to it.
    Detach and delete the disk image container
    Delete the vm image container and the VM.
    Create a vm image container, and the VM.
    Check the VM for provision succedded:
        if so, exit
        if provisioning failed:
            delete the vm, delete the vm image, create a disk image
            from the VM's vhd, attach the disk to the stable VM, copy
            /var/log and /var/lib/waagent to the localhost.
    """
    stableVM=''
    mountOptions=''
    stableVMMountpoint='/mnt/disk' # need to ensure this is created if not existing.
    sourceVHD=''
    location=""
    localAgent='/home/ericg/Public/git_repos/private/WALinuxAgent-Private/waagent_freebsd'
    localInfo='./logs'
    stableVMaccount='' # Need to ensure root permissions for this to work
    stableVMpass=''
    stableVMCert=''
    provisionedVMaccount=''
    provisionedVMpass=''
    provisionedVMCert=''
    account=''
    testname='azuretest'
    partNum=1
    provision_retries=1
    fstype='scsi'
    keep_vhd='no'
    teardown_test_vm='fail'
    teardown_stable_vm='fail'
    prompt = 'yes'
    stable_vm_vhd=None
    stableVMImageName=None
    stable_vm_image=None
    logfile='azure_test.log'
    """
    We need to create a disk image container and attach it to a stable
    vm in order to copy the current sources to it.  Then we detach it,
    delete the disk image container, create a vm image container, and
    the VM. 
    """

    for i in range(len(sys.argv)) :
        if '--stable_vm' == sys.argv[i] : stableVM=sys.argv[i+1]
        elif '--source_disk' == sys.argv[i]: sourceVHD=sys.argv[i+1]
        elif '--storage_acct' == sys.argv[i]: account=sys.argv[i+1]
        elif '--testname' == sys.argv[i] : testname=sys.argv[i+1]
        elif '--stable_vm_mount_point' == sys.argv[i] : stableVMMountpoint=sys.argv[i+1]
        elif '--agent_path' == sys.argv[i] : localAgent=sys.argv[i+1]
        elif '--stable_vm_acct_name' == sys.argv[i] : stableVMaccount=sys.argv[i+1]
        elif '--stable_vm_acct_pass' == sys.argv[i] : stableVMpass=sys.argv[i+1]
        elif '--stable_vm_acct_cert' == sys.argv[i] : stableVMCert=sys.argv[i+1]
        elif '--test_vm_acct_name' == sys.argv[i] : provisionedVMaccount=sys.argv[i+1]
        elif '--test_vm_acct_pass' == sys.argv[i] : provisionedVMpass=sys.argv[i+1]
        elif '--test_vm_acct_cert' == sys.argv[i] : provisionedVMCert=sys.argv[i+1]
        elif '--azure_location' == sys.argv[i] : location=sys.argv[i+1]
        elif '--mount_opts' == sys.argv[i] : mountOptions=sys.argv[i+1]
        elif '--part_num' == sys.argv[i] : partNum=sys.argv[i+1]
        elif '--retries' == sys.argv[i] : provision_retries=int(sys.argv[i+1])
        elif '--fs_type' == sys.argv[i] : fstype=sys.argv[i+1]
        elif '--keep_test_vm_vhd' == sys.argv[i] : keep_vhd=sys.argv[i+1]
        elif '--teardown_test_vm' == sys.argv[i] : teardown_test_vm=sys.argv[i+1]
        elif '--teardown_stable_vm' == sys.argv[i] : teardown_stable_vm=sys.argv[i+1]
        elif '--prompt' == sys.argv[i] : prompt=sys.argv[i+1]
        elif '--stable_vm_image' == sys.argv[i] : stable_vm_image=sys.argv[i+1]
        elif '--stable_vm_vhd' == sys.argv[i] : stable_vm_vhd=sys.argv[i+1]
        elif '--logfile' == sys.argv[i] : logfile=sys.argv[i+1]
        
    LoggerInit(logfile,'')
    waagent.Log("User: "+ pwd.getpwuid(os.geteuid()).pw_name +" Running Command :\n" + reduce(lambda x, y: x+' '+y,sys.argv))

    if len(stableVM) == 0 and not ( stable_vm_image or stable_vm_vhd ):
        print '--vm <stable vm> must be provided unless --stable_vm_image or --stable_vm_vhd'
        waagent.Log( '--vm <stable vm> must be provided!')
        sys.exit(1)
    else:
        if stable_vm_image:
            sourceVHD=createStableVMFromVMImage(stable_vm_image,sourceVHD)
            stableVM=testname+'-stable-vm'
        elif stable_vm_vhd:
            stableVMImageName=testname+'-stable-vi'
            sourceVHD=createStableVMFromVHD(stableVMImageName,stable_vm_vhd)
            stableVM=testname+'-stable-vm'
        p = False
        retries = provision_retries
        while not p and retries > 0:
            p,out = checkVMProvisioned(stableVM)
            if not p:
                if 'Failed' in out or 'Timeout' in out :
                    break
                print  stableVM + ' Not Provisioned - sleeping on retry:' + str( provision_retries - retries ) 
                waagent.Log(  stableVM + ' Not Provisioned - sleeping on retry:' + str( provision_retries - retries ) )
                time.sleep(30)
                retries -= 1
            else :
                print stableVM + ' Provision SUCCEEDED.'
                waagent.Log( stableVM + ' Provision SUCCEEDED.')
    # done creating the stable vm        
    vmImageName=os.path.splitext(os.path.basename(sourceVHD))[0]+'-vi'
    #flushVMImage(vmImageName)

    # if no disk image name is provided we want to clone the stable vm disk.
    if not sourceVHD:
        sourceVHD=createDiskImageFromStableVMDisk(stableVM)
        if not sourceVHD:
            print 'Errors - unable to create disk image - assuming created'
            waagent.Log( 'Errors - unable to create disk image - assuming created')
        diskImageName=os.path.splitext(os.path.basename(sourceVHD))[0]
    else :
        diskImageName=os.path.splitext(os.path.basename(sourceVHD))[0]+'-di'
        diskImageVHD,code=makeDiskImage(diskImageName,sourceVHD,location,True)
        if code:
            print 'Error - unable to make ' + diskImageName
            waagent.Log( 'Error - unable to make ' + diskImageName)

    lun=addDiskImageToVM(stableVM,diskImageName)
    while lun == None :
        time.sleep(2)
        lun=addDiskImageToVM(stableVM,diskImageName)

    doPrompt()
    updateAgent(localAgent,stableVM,stableVMaccount,stableVMCert,stableVMMountpoint,mountOptions,lun,partNum,provisionedVMaccount)
    doPrompt()
    #reboot to prevent stale mount bugs
    cmd= 'export SUDO_ASKPASS=./pw.sh && sudo -A reboot'
    ssh_command(stableVM,stableVMaccount,cmd)

    while dropDiskImageFromVM(stableVM,lun)[1] != 0 :
        time.sleep(2)
    while flushDiskImage(diskImageName,False)[1] != 0 :
        time.sleep(2)
    vmImageName=os.path.splitext(os.path.basename(sourceVHD))[0]+'-vi'
    flushVMImage(vmImageName)
    vmName=testname+'-vm'
    flushVM(vmName)
    makeVMImage(vmImageName,diskImageVHD,True)
    sourceVHD=makeVM(vmName,vmImageName,sourceVHD,testname,location,provisionedVMaccount,provisionedVMpass,provisionedVMCert,True)
    print 'The new source vhd is ' + sourceVHD
    waagent.Log( 'The new source vhd is ' + sourceVHD)
    p = False
    retries = provision_retries
    while not p and retries > 0:
        p,out = checkVMProvisioned(vmName)
        if not p:
            if 'Failed' in out or 'Timeout' in out :
                break
            print  vmName + ' Not Provisioned - sleeping on retry:' + str( provision_retries - retries ) 
            waagent.Log(  vmName + ' Not Provisioned - sleeping on retry:' + str( provision_retries - retries ) )
            time.sleep(30)
        else :
            print vmName + ' Provision SUCCEEDED.'
            waagent.Log( vmName + ' Provision SUCCEEDED.')
            doPrompt()
            if teardown_test_vm in ('success','always'):
                teardown(testname+'_pass')
            sys.exit(0)
        retries -= 1
    
    print vmName + ' Provision FAILED.'
    waagent.Log( vmName + ' Provision FAILED.')
    doPrompt()
    if teardown_test_vm in ('fail','always'):
                teardown(testname+'_fail')
    sys.exit(1)



#!/usr/bin/python
#
# Windows Azure Linux Agent setup.py
#
# Copyright 2013 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import glob
import os
import sys
import platform
import setuptools
from setuptools.command.install import install

from distutils.errors import DistutilsArgError

def getDistro():
    """
    Try to figure out the distribution we are running on
    """
    distro = platform.linux_distribution()[0].lower()
    # Manipulate the distribution to meet our needs we treat
    # Fedora, RHEL, and CentOS the same
    # openSUSE and SLE the same
    if distro.find('suse') != -1:
        distro = 'suse'
    if (distro.find('fedora') != -1
    or distro.find('red hat') != -1
    or distro.find('centos') != -1):
        distro = 'redhat'

    return distro

class InstallData(install):
    user_options = install.user_options + [
        # This will magically show up in member variable 'init_system'
        ('init-system=', None, 'init system to configure [default: sysV]'),
        ('lnx-distro=', None, 'target Linux distribution'),
    ]

    def initialize_options(self):
        install.initialize_options(self)
        self.init_system = 'sysV'
        self.lnx_distro = None

    def finalize_options(self):
        install.finalize_options(self)
        if not self.lnx_distro:
            self.lnx_distro = getDistro()
        if self.init_system not in ['sysV', 'systemd', 'upstart']:
            print 'Do not know how to handle %s init system' %self.init_system
            sys.exit(1)
        if self.init_system == 'sysV':
            if not os.path.exists('distro/%s' %self.lnx_distro):
                msg = 'Unknown distribution "%s"' %self.lnx_distro
                msg += ', no entry in distro directory'
                sys.exit(1)

    def run(self):
        """
        Install the files for the Windows Azure Linux Agent
        """
        distro = self.lnx_distro
        init = self.init_system
        prefix = self.prefix
        tgtDir = self.root
        if prefix and prefix[-1] != '/':
            prefix += '/'
        else:
            prefix = '/'
        if tgtDir and tgtDir[-1] != '/':
            tgtDir += '/'
        else:
            tgtDir = '/'
        # Handle the different init systems
        if init == 'sysV':
            initdir = 'etc/init.d'
            if self.lnx_distro == 'redhat':
                initdir = 'etc/rc.d/init.d'
            if not os.path.exists(tgtDir + initdir):
                try:
                    self.mkpath(tgtDir + initdir, 0755)
                except:
                    msg = 'Could not create init script directory '
                    msg += tgtDir
                    msg += initdir
                    print msg
                    print sys.exc_info()[0]
                    sys.exit(1)
            initScripts = glob.glob('distro/%s/*.sysV' %distro)
            try:
                for f in initScripts:
                    newName = f.split('/')[-1].split('.')[0]
                    self.copy_file(f, tgtDir + initdir + '/' + newName)
            except:
                print 'Could not install systemV init script', 
                sys.exit(1)
        elif init == 'systemd':
            if not os.path.exists(tgtDir + prefix +'lib/systemd/system'):
                try:
                    self.mkpath(tgtDir + prefix + 'lib/systemd/system', 0755)
                except:
                    msg = 'Could not create systemd service directory '
                    msg += tgtDir + prefix
                    msg += 'lib/systemd/system'
                    print msg
                    sys.exit(1)
            services = glob.glob('distro/systemd/*')
            for f in services:
                try:
                    baseName = f.split('/')[-1]
                    self.copy_file(f,
                            tgtDir + prefix +'lib/systemd/system/' + baseName)
                except:
                    print 'Could not install systemd service files'
                    sys.exit(1)
        elif init == 'upstart':
            print 'Upstart init files installation not supported at this time.'
            print 'Need an implementation, please submit a patch ;)'
            print 'See WALinuxAgent/debian directory for Debian/Ubuntu packaging'
    
        # Configuration file
        if not os.path.exists(tgtDir + 'etc'):
            try:
                self.mkpath(tgtDir + 'etc', 0755)
            except:
                msg = 'Could not create config dir '
                msg += tgtDir
                msg += 'etc'
                print msg
                sys.exit(1)
        try:
            self.copy_file('config/waagent.conf', tgtDir + 'etc/waagent.conf')
        except:
            print 'Could not install configuration file %setc' %tgtDir
            sys.exit(1)

        if not os.path.exists(tgtDir + 'etc/udev/rules.d'):
            try:
                self.mkpath(tgtDir + 'etc/udev/rules.d', 0755)
            except Exception as e:
                print e

        try:
            self.copy_file('config/99-azure-product-uuid.rules', tgtDir + 'etc/udev/rules.d/99-azure-product-uuid.rules')
        except Exception as e:
            print e
            print 'Could not install product uuid rules file %setc' %tgtDir
            sys.exit(1)

        if not os.path.exists(tgtDir + 'etc/logrotate.d'):
            try:
                self.mkpath(tgtDir + 'etc/logrotate.d', 0755)
            except:
                msg = 'Could not create ' + tgtDir + 'etc/logrotate.d'
                print msg
                sys.exit(1)
        try:
            self.copy_file('config/waagent.logrotate',
                      tgtDir + 'etc/logrotate.d/waagent')
        except:
            msg = 'Could not install logrotate file in '
            msg += tgtDir + 'etc/logrotate.d'
            print  msg
            sys.exit(1)
         
        # Daemon
        if not os.path.exists(tgtDir + prefix + 'sbin'):
            try:
                self.mkpath(tgtDir + prefix + 'sbin', 0755)
            except:
                msg = 'Could not create target daemon dir '
                msg+= tgtDir + prefix + 'sbin'
                print msg
                sys.exit(1)
        try:
            self.copy_file('waagent', tgtDir + prefix + 'sbin/waagent')
        except:
            print 'Could not install daemon %s%ssbin/waagent' %(tgtDir,prefix)
            sys.exit(1)
        os.chmod('%s%ssbin/waagent' %(tgtDir,prefix), 0755)

def readme():
    with open('README') as f:
        return f.read()
    
setuptools.setup(name = 'waagent',
      version = '1.4.0',
      description = 'Windows Azure Linux Agent',
      long_description = readme(),
      author = 'Stephen Zarkos, Eric Gable',
      author_email = 'walinuxagent@microsoft.com',
      platforms = 'Linux',
      url = 'https://github.com/Windows-Azure/',
      license = 'Apache License Version 2.0',
      cmdclass = {
          # Use a subclass for install that handles
          # install, we do not have a "true" python package
          'install': InstallData,
      },
)






# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import unittest
from env import waagent
import sys
from tests.tools import *

class MockHTTPResponse(object):
    def __init__(self, status=200):
        self.status = status
        self.reason = "foo"

    def getheaders(*args, **kwargs):
        return {"hehe" : "haha"}

    def read(*args, **kwargs):
        return "bar"

class MockOldHTTPConnection(object):
    MockHost=None
    MockPort=None
    MockUrl=None
    MockCallCount=0

    def __init__(self, host, port):
        self.__class__.MockHost = host
        self.__class__.MockPort = port

    def request(self, method, url, data, headers = None):
        self.__class__.MockUrl = url
        self.__class__.MockCallCount += 1
    
    def getresponse(*args, **kwargs):
        return MockHTTPResponse()

class MockHTTPConnection(MockOldHTTPConnection):
    def set_tunnel(*args, **kwargs):
        pass

class MockBadHTTPConnection(MockHTTPConnection):
    def getresponse(*args, **kwargs):
        return MockHTTPResponse(500)

class MockHttpLib(object):
    def __init__(self):
        self.HTTPConnection = MockHTTPConnection
        self.OK = 200

MockOSEnv = {
        "http_proxy":"http://httpproxy:8888",
        "https_proxy":"https://httpsproxy:8888"
}

class TestHttp(unittest.TestCase):

    def test_parseurl(self):
        httputil = waagent.Util()
        host, port, secure, path = httputil._ParseUrl("http://foo:8/bar?hehe")
        self.assertEquals("foo", host)
        self.assertEquals(8, port)
        self.assertEquals(False, secure)
        self.assertEquals("/bar?hehe", path)
        
        host, port, secure, path = httputil._ParseUrl("http://foo.bar/")
        self.assertEquals("foo.bar", host)
        self.assertEquals(80, port)
        self.assertEquals(False, secure)
        self.assertEquals("/", path)

        host, port, secure, path= httputil._ParseUrl("https://foo.bar/")
        self.assertEquals("foo.bar", host)
        self.assertEquals(80, port)
        self.assertEquals(True, secure)
        self.assertEquals("/", path)

        self.assertRaises(ValueError, httputil._ParseUrl, 
                          "https://a:b@foo.bar/")

        host, port, secure, path = httputil._ParseUrl("https://foo.bar")
        self.assertEquals("foo.bar", host)
        self.assertEquals(80, port)
        self.assertEquals(True, secure)
        self.assertEquals("/", path)

        host, port, secure, path = httputil._ParseUrl("http://a:b@foo.bar:8888")
        self.assertEquals("a:b@foo.bar", host)
        self.assertEquals(8888, port)
        self.assertEquals(False, secure)
        self.assertEquals("/", path)
    
    @Mockup(waagent.httplib, "HTTPConnection", MockHTTPConnection)
    @Mockup(waagent.os, "environ", MockOSEnv)
    def test_http_request(self):
        httputil = waagent.Util()

        #If chkProxy is on, host and port should point to proxy server
        httputil.HttpRequest("GET", "http://foo.bar/get", chkProxy=True)
        self.assertEquals("httpproxy", MockHTTPConnection.MockHost) 
        self.assertEquals(8888, MockHTTPConnection.MockPort) 
        self.assertEquals("http://foo.bar:80/get", MockHTTPConnection.MockUrl) 
        
        #If chkProxy is off, ignore proxy
        httputil.HttpRequest("GET", "http://foo.bar/get", chkProxy=False)
        self.assertEquals("foo.bar", MockHTTPConnection.MockHost) 
        self.assertEquals(80, MockHTTPConnection.MockPort) 
        self.assertEquals("/get", MockHTTPConnection.MockUrl) 

    @Mockup(waagent, "httplib" , MockHttpLib())
    def test_https_fallback(self):
        httputil = waagent.Util()
        print "The bellowing warning log is expected:"
        httputil.HttpRequest("GET", "https://foo.bar/get")
        self.assertEquals("/get", MockHTTPConnection.MockUrl)

    @Mockup(waagent.httplib, "HTTPConnection", MockOldHTTPConnection)
    @Mockup(waagent.httplib, "HTTPSConnection", MockOldHTTPConnection)
    @Mockup(waagent.os, "environ", MockOSEnv)
    def test_https_fallback2(self):
        httputil = waagent.Util()
        print "The bellowing warning log is expected:"
        httputil.HttpRequest("GET", "https://foo.bar/get", chkProxy=True)
        self.assertEquals("http://foo.bar:80/get", MockOldHTTPConnection.MockUrl)

    @Mockup(waagent.Util, "RetryWaitingInterval", 0)
    @Mockup(waagent.httplib, "HTTPConnection", MockBadHTTPConnection)
    def test_retry(self):
        httputil = waagent.Util()
        MockBadHTTPConnection.MockCallCount=0
        print "The bellowing error log is expected:"
        httputil.HttpRequest("GET", "http://foo.bar", chkProxy=False, maxRetry=1)
        self.assertEquals(2, MockBadHTTPConnection.MockCallCount)
    
if __name__ == '__main__':
    unittest.main()



# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os
import re
import unittest
from env import waagent

class MockDistro(object):
    def getInterfaceNameByMac(self, mac):
        pass

    def configIpV4(self, ifName, addr):
        pass

class TestSharedConfig(unittest.TestCase):

    def test_reg(self):
        mac = "00:15:5D:34:00:08"
        output = Ifconfig_Out
        output = output.replace('\n', '')
        reg = r"(eth\d).*(HWaddr|ether) {0}".format(mac)
        match = re.search(reg, output, re.IGNORECASE)
        output = match.group(0)
        eths = re.findall(r"eth\d", output)
        self.assertNotEquals(0, len(eths))
    
    def test_parse_shared_config(self):
        conf = waagent.SharedConfig().Parse(SharedConfigText)
        self.assertNotEquals(None, conf)
        self.assertNotEquals(None, conf.RdmaMacAddress)
        self.assertNotEquals(None, conf.RdmaIPv4Address)
        self.assertEquals("00:15:5D:34:00:44", conf.RdmaMacAddress)
        return conf

    def test_config_rdma(self):
        waagent.MyDistro= MockDistro()
        waagent.LibDir="/tmp"

        test_dev = "/tmp/hvnd_rdma"
        test_dat_conf_files = ["/tmp/dat.conf"]
        if os.path.isfile("/tmp/rdmaconfiged"):
            os.remove("/tmp/rdmaconfiged")
        waagent.SetFileContents(test_dev, "")
        old = ("ofa-v2-ib0 u2.0 nonthreadsafe default libdaplofa.so.2 "
               "dapl.2.0 \"oldip 0\"")
        waagent.SetFileContents(test_dat_conf_files[0], old)
        conf = self.test_parse_shared_config()
        handler = waagent.RdmaHandler(conf.RdmaMacAddress, conf.RdmaIPv4Address,
                                      test_dev, test_dat_conf_files)
        handler.set_dat_conf()
        handler.set_rdma_dev()

        rdma_conf = waagent.GetFileContents(test_dev)
        self.assertNotEquals(None, rdma_conf)
        self.assertNotEquals(0, rdma_conf.count(conf.RdmaIPv4Address))
        self.assertNotEquals(0, rdma_conf.count(conf.RdmaMacAddress))

        dat_conf = waagent.GetFileContents(test_dat_conf_files[0])
        self.assertNotEquals(None, dat_conf)
        self.assertNotEquals(0, dat_conf.count(conf.RdmaIPv4Address))
        self.assertEquals(0, dat_conf.count("oldip"))

SharedConfigText="""\
<?xml version="1.0" encoding="utf-8"?>
<SharedConfig version="1.0.0.0" goalStateIncarnation="1">
  <Deployment name="698f959e434c41cc9d72a2c67c044463" guid="{ba92e945-0302-4030-9710-257c03c07e22}" incarnation="0" isNonCancellableTopologyChangeEnabled="false">
    <Service name="test-rdms" guid="{00000000-0000-0000-0000-000000000000}" />
    <ServiceInstance name="698f959e434c41cc9d72a2c67c044463.0" guid="{6f157bcb-b6ac-4fdd-9789-2ca466220e17}" />
  </Deployment>
  <Incarnation number="1" instance="test-rdms" guid="{33d19bb6-f34d-4dfb-966c-2bade1714cc5}" />
  <Role guid="{dad0becc-5d1d-3c55-3285-0136e9933bbe}" name="test-rdms" settleTimeSeconds="0" />
  <LoadBalancerSettings timeoutSeconds="0" waitLoadBalancerProbeCount="8">
    <Probes>
      <Probe name="D41D8CD98F00B204E9800998ECF8427E" />
      <Probe name="423A4BBA20CEBE79BA641B20A03ED6F9" />
    </Probes>
  </LoadBalancerSettings>
  <OutputEndpoints>
    <Endpoint name="test-rdms:openInternalEndpoint" type="SFS">
      <Target instance="test-rdms" endpoint="openInternalEndpoint" />
    </Endpoint>
  </OutputEndpoints>
  <Instances>
    <Instance id="test-rdms" address="100.74.58.20" primaryMacAddress="000D3A101ED4" rdmaMacAddress="00155D340044" rdmaIPv4Address="172.16.2.59">
      <FaultDomains randomId="0" updateId="0" updateCount="0" />
      <InputEndpoints>
        <Endpoint name="openInternalEndpoint" address="100.74.58.20" protocol="any" isPublic="false" enableDirectServerReturn="false" isDirectAddress="false" disableStealthMode="false">
          <LocalPorts>
            <LocalPortSelfManaged />
          </LocalPorts>
        </Endpoint>
        <Endpoint name="SSH" address="100.74.58.20:22" protocol="tcp" hostName="test-rdmsContractContract" isPublic="true" loadBalancedPublicAddress="104.45.128.35:22" enableDirectServerReturn="false" isDirectAddress="false" disableStealthMode="false">
          <LocalPorts>
            <LocalPortRange from="22" to="22" />
          </LocalPorts>
        </Endpoint>
        <Endpoint name="test-rdms_A9_Infiniband" address="100.74.58.20" protocol="any" isPublic="false" enableDirectServerReturn="false" isDirectAddress="false" disableStealthMode="false">
          <LocalPorts>
            <LocalPortSelfManaged />
          </LocalPorts>
        </Endpoint>
      </InputEndpoints>
    </Instance>
  </Instances>
</SharedConfig>
"""
Ifconfig_Out="""\
eth0: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
inet 100.74.52.8  netmask 255.255.255.0  broadcast 100.74.52.255
inet6 fe80::20d:3aff:fe10:672f  prefixlen 64  scopeid 0x20<link>
ether 00:0d:3a:10:67:2f  txqueuelen 1000  (Ethernet)
RX packets 9911  bytes 4451278 (4.2 MiB)
RX errors 0  dropped 0  overruns 0  frame 0
TX packets 10505  bytes 1643251 (1.5 MiB)
TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0

eth1: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
inet6 fe80::215:5dff:fe34:8  prefixlen 64  scopeid 0x20<link>
ether 00:15:5d:34:00:08  txqueuelen 1000  (Ethernet)
RX packets 16  bytes 672 (672.0 B)
RX errors 0  dropped 0  overruns 0  frame 0
TX packets 16  bytes 2544 (2.4 KiB)
TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0

lo: flags=73<UP,LOOPBACK,RUNNING>  mtu 65536
inet 127.0.0.1  netmask 255.0.0.0
inet6 ::1  prefixlen 128  scopeid 0x10<host>
loop  txqueuelen 0  (Local Loopback)
RX packets 0  bytes 0 (0.0 B)
RX errors 0  dropped 0  overruns 0  frame 0
TX packets 0  bytes 0 (0.0 B)
"""

if __name__ == '__main__':
    unittest.main()



# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import imp
import os
import sys

project_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
waagent = imp.load_source('waagent', os.path.join(project_root, 'waagent'))
sys.path.insert(0, project_root)

waagent.LoggerInit('/dev/stdout', '/dev/null')

